import os
import csv
import copy
import numpy as np
from .classes import *
from .utils import get_encoding
from shapely import geometry

def readNet(node_filepath,link_filepath,poi_filepath,zone_filepath,demand_filepath,poi_trip_filepath,agent_filepath):
    network=Network()

    #read network nodes
    if os.path.exists(node_filepath):
        try:
            num_node=0
            x_coords=[]
            y_coords=[]

            encode=get_encoding(node_filepath)
            node_reader=csv.DictReader(open(node_filepath,'r',encoding=encode,errors='ignore'))
            for row_id,row in enumerate(node_reader):
                node=Node(row,row_id)
                network.node_dict[node.node_id]=node
                x_coords.append(node.x_coord)
                y_coords.append(node.y_coord)
                num_node += 1

            network.number_of_node=num_node
            network.node_coords=[x_coords,y_coords]
            network.min_lng,network.max_lng=min(x_coords),max(x_coords)
            network.min_lat,network.max_lat=min(y_coords),max(y_coords)
        except Exception as e:
            print("something is wrong when reading {}:".format(node_filepath))
            print(e)
    else:
        print("error: node.csv doesn't exist")
        sys.exit(0)

    #read network links
    if os.path.exists(link_filepath):
        try:
            num_link=0
            unique_links=[]
            encode=get_encoding(link_filepath)
            link_reader = csv.DictReader(open(link_filepath,'r',encoding=encode,errors='ignore'))
            for row_id,row in enumerate(link_reader):
                link=Link(row,row_id)
                if link.link_id==None:
                    link.link_id=num_link
                network.link_dict[link.link_id] = link
                try:
                    network.node_dict[link.from_node_id].out_link_list.append(link)
                    network.node_dict[link.to_node_id].in_link_list.append(link)
                except:
                    pass
                if not link.geometry:
                    from_node=network.node_dict[link.from_node_id]
                    to_node=network.node_dict[link.to_node_id]
                    link.geometry=geometry.LineString([(from_node.x_coord,from_node.y_coord),(to_node.x_coord,to_node.y_coord)])
                if (link.from_node_id,link.to_node_id) not in unique_links and (link.to_node_id,link.from_node_id) not in unique_links:
                    coords = list(link.geometry.coords)
                    network.link_coords.append(np.array(coords))
                    unique_links.append((link.from_node_id, link.to_node_id))
                    num_link += 1
            network.number_of_link=num_link
        except Exception as e:
            print("something is wrong when reading {}:".format(link_filepath))
            print(e)
    else:
        print("error: link.csv doesn't exist")
        sys.exit(0)

    if os.path.exists(poi_filepath):
        try:
            encode=get_encoding(poi_filepath)
            poi_reader = csv.DictReader(open(poi_filepath,'r',encoding=encode,errors='ignore'))
            poi_id=0
            for row_id,row in enumerate(poi_reader):
                poi=POI(row,row_id)
                if poi.poi_id==None:
                    poi.poi_id=poi_id
                network.poi_dict[poi.poi_id]=poi
                if isinstance(poi.geometry,geometry.MultiPolygon):
                    for geom in poi.geometry.geoms:
                        coords = list(geom.exterior.coords)
                        network.poi_coords.append(np.array(coords))
                        poi_id += 1
                elif isinstance(poi.geometry,geometry.Polygon):
                    coords=list(poi.geometry.exterior.coords)
                    network.poi_coords.append(np.array(coords))
                    poi_id+=1
            network.number_of_poi=poi_id
        except Exception as e:
            print("something is wrong when reading {}:".format(poi_filepath))
            print(e)
    else:
        print("warning: poi.csv doesn't exist")

    # read zone
    if os.path.exists(zone_filepath):
        try:
            zone_id=0
            min_zone_id=float('inf')
            max_zone_id=-float('inf')
            encode=get_encoding(zone_filepath)
            zone_reader = csv.DictReader(open(zone_filepath, 'r',encoding=encode,errors='ignore'))
            for row_id,row in enumerate(zone_reader):
                zone=Zone(row,row_id)
                network.zone_dict[zone.activity_zone_id]=zone
                zone_id+=1
                if zone.activity_zone_id>max_zone_id:
                    max_zone_id=zone.activity_zone_id
                if zone.activity_zone_id<min_zone_id:
                    min_zone_id=zone.activity_zone_id
            network.number_of_zone=zone_id
            network.range_of_zone_ids=[min_zone_id,max_zone_id]
        except Exception as e:
            print("something is wrong when reading {}:".format(zone_filepath))
            print(e)
    else:
        print("warning: zone.csv doesn't exist")
    # read demand
    if os.path.exists(demand_filepath):
        try:
            demand_id = 0
            encode=get_encoding(demand_filepath)
            demand_reader = csv.DictReader(open(demand_filepath, 'r',encoding=encode,errors='ignore'))
            for row_id,row in enumerate(demand_reader):
                demand = Demand(row,row_id)
                network.demand_dict[demand_id] = demand
                demand_id += 1
            network.number_of_demand = demand_id
        except Exception as e:
            print("something is wrong when reading {}:".format(demand_filepath))
            print(e)
    else:
        print("warning: demand.csv doesn't exist")
    # read poi_trip
    if os.path.exists(poi_trip_filepath):
        try:
            poi_type_id=0
            encode=get_encoding(poi_trip_filepath)
            poi_trip_reader = csv.DictReader(open(poi_trip_filepath,'r',encoding=encode,errors='ignore'))
            for row_id,row in enumerate(poi_trip_reader):
                poi_trip_=POITrip(row,row_id)
                for building in poi_trip_.building:
                    poi_trip=copy.deepcopy(poi_trip_)
                    poi_trip.building=building
                    network.poi_trip_dict[building]=poi_trip
                    poi_type_id+=1
            network.number_of_poi_type=poi_type_id
        except Exception as e:
            print("something is wrong when reading {}:".format(poi_trip_filepath))
            print(e)
    else:
        print("warning: poi_trip_rate.csv doesn't exist")
    # read agent
    if os.path.exists(agent_filepath):
        try:
            agent_id=0
            encode=get_encoding(agent_filepath)
            agent_reader = csv.DictReader(open(agent_filepath, 'r',encoding=encode,errors='ignore'))
            for row_id,row in enumerate(agent_reader):
                agent=Agent(row,row_id)
                if agent.agent_id:
                    network.agent_dict[agent.agent_id] = agent
                    agent_id += 1
            network.number_of_agent = agent_id
        except Exception as e:
            print("something is wrong when reading {}:".format(agent_filepath))
            print(e)
    else:
        print("warning: input_agent.csv doesn't exist")
    return network



import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from matplotlib.collections import LineCollection
from matplotlib.collections import PolyCollection
from scipy.interpolate import griddata as gd
from matplotlib.lines import Line2D


def plotNetbySelectedObj(node_coords,link_coords,poi_coords,savefig):
    fig, ax = plt.subplots(figsize=(10,8))
    # plot network nodes
    ax.scatter(node_coords[0], node_coords[1], marker='o', c='red', s=10,zorder=1)
    # plot network links
    ax.add_collection(LineCollection(link_coords, colors='orange', linewidths=1,zorder=2))
    # plot network pois
    if len(poi_coords):
        coll = PolyCollection(poi_coords, alpha=0.7,zorder=0)
        ax.add_collection(coll)
    # set axis
    ax.autoscale_view()
    plt.xlabel('x_coord')
    plt.ylabel('y_coord')
    plt.tight_layout()
    # show fig
    plt.show()
    #save fig
    if savefig:
        try:
            figname=savefig['filename'] if 'filename' in savefig.keys() else 'mode_network.png'
            dpi=savefig['dpi'] if 'dpi' in savefig else 300
            fig.savefig(figname,dpi=dpi,bbox_inches='tight')
        except Exception as e:
            print(e)
def plotNetbyNodeAttrRange(node_coords, link_coords,poi_coords,attr,value,savefig):
    fig, ax = plt.subplots(figsize=(10, 8))
    # plot network nodes
    max_v,min_v=max(value),min(value)
    s=np.array(value)/max_v*95+5
    ax.scatter(node_coords[0], node_coords[1], marker='o', c='red', s=s,zorder=2)
    # plot network links
    ax.add_collection(LineCollection(link_coords, colors='orange', linewidths=1, zorder=1))
    # plot network pois
    coll = PolyCollection(poi_coords, alpha=0.5, zorder=0)
    ax.add_collection(coll)
    # add legend
    proxies = []
    labels = []
    min_index = value.index(min_v)
    ax2 = ax.scatter(node_coords[0][min_index], node_coords[1][min_index], marker='o', c='red', s=5, zorder=2)
    proxies.append(ax2)
    labels.append('%s:%.4f' %(attr,min_v))
    max_index = value.index(max_v)
    ax1 = ax.scatter(node_coords[0][max_index], node_coords[1][max_index], marker='o', c='red', s=100, zorder=2)
    proxies.append(ax1)
    labels.append('%s:%.4f' % (attr, max_v))
    ax.legend(proxies, labels,loc='upper right')
    # set axis
    ax.autoscale_view()
    plt.xlabel('x_coord')
    plt.ylabel('y_coord')
    plt.tight_layout()
    # show fig
    plt.show()
    # save fig
    if savefig:
        try:
            figname = savefig['filename'] if 'filename' in savefig.keys() else 'node_attr_range.png'
            dpi = savefig['dpi'] if 'dpi' in savefig else 300
            fig.savefig(figname, dpi=dpi, bbox_inches='tight')
        except Exception as e:
            print(e)
def plotNetbyLinkAttrRange(node_coords, link_coords,poi_coords,attr,value,savefig):
    fig, ax = plt.subplots(figsize=(10, 8))
    # plot network nodes
    ax.scatter(node_coords[0], node_coords[1], marker='o', c='red', s=10, zorder=2)
    # plot network links
    max_v, min_v = max(value), min(value)
    w= np.array(value) / max_v *4.5 + 0.5
    ax.add_collection(LineCollection(link_coords, colors='orange', linewidths=w, zorder=1))
    # plot network pois
    coll = PolyCollection(poi_coords, alpha=0.5, zorder=0)
    ax.add_collection(coll)
    # add legend
    proxies = [Line2D([0, 1], [0, 1], color='orange', linewidth=0.5),
               Line2D([0, 1], [0, 1], color='orange', linewidth=5)]
    ax.legend(proxies, ['%s:%.4f'%(attr,min(value)), '%s:%.4f'%(attr,max(value))],loc='upper right')
    # set axis
    ax.autoscale_view()
    plt.xlabel('x_coord')
    plt.ylabel('y_coord')
    plt.tight_layout()
    # show fig
    plt.show()
    # save fig
    if savefig:
        try:
            figname = savefig['filename'] if 'filename' in savefig.keys() else 'link_attr_range.png'
            dpi = savefig['dpi'] if 'dpi' in savefig else 300
            fig.savefig(figname, dpi=dpi, bbox_inches='tight')
        except Exception as e:
            print(e)
def plotNetbyZonePoiAttrDensity(poi_coords,value,savefig):
    fig, ax = plt.subplots(figsize=(10,8))
    # plot network pois
    coll = PolyCollection(poi_coords, array=np.array(value), cmap='jet', edgecolors='none')
    ax.add_collection(coll)
    fig.colorbar(coll, ax=ax)
    # set axis
    ax.autoscale_view()
    plt.xlabel('x_coord')
    plt.ylabel('y_coord')
    plt.tight_layout()
    # show fig
    plt.show()
    # save fig
    if savefig:
        try:
            figname = savefig['filename'] if 'filename' in savefig.keys() else 'zone_poi_attr_density.png'
            dpi = savefig['dpi'] if 'dpi' in savefig else 300
            fig.savefig(figname, dpi=dpi, bbox_inches='tight')
        except Exception as e:
            print(e)

def plotNetbyZoneDeamndHeat(network,demand,annot,savefig):

    max_vol = np.max(demand.reshape(1, -1))
    min_vol = np.min(demand.reshape(1, -1))
    labels = [str(i + 1) for i in range(network.number_of_zone)]
    data = pd.DataFrame(demand, index=labels, columns=labels)
    fig=sns.heatmap(data=data, vmax=max_vol, vmin=min_vol, annot=annot, cmap='jet')
    # set axis
    plt.xlabel('to_zone_id')
    plt.ylabel('from_zone_id')
    plt.tight_layout()
    # show fig
    plt.show()
    # save fig
    if savefig:
        try:
            figname = savefig['filename'] if 'filename' in savefig.keys() else 'zone_demand_density.png'
            dpi = savefig['dpi'] if 'dpi' in savefig else 300
            scatter_fig = fig.get_figure()
            scatter_fig.savefig(figname, dpi=dpi, bbox_inches='tight')
        except Exception as e:
            print(e)
def plotNetbyZoneDemandFlow(network,demand_line,values, zone_grid, zone_labels,annot,bg,savefig):
    fig, ax = plt.subplots(figsize=(10, 8))
    if bg:
        # plot network links
        ax.add_collection(LineCollection(network.link_coords, colors='black', linewidths=1, zorder=1))
        # plot network pois
        coll = PolyCollection(network.poi_coords, alpha=0.5, zorder=0)
        ax.add_collection(coll)
    # plot network demand flow
    w=np.array(values)/max(values)*4.5+0.5
    ax.add_collection(LineCollection(demand_line, colors='orange', linewidths=w, zorder=2))

    # plot network zone
    coll = PolyCollection(zone_grid,facecolors='none',linewidths=5,edgecolors='olive', zorder=0)
    ax.add_collection(coll)
    if annot:
        for label in zone_labels:
            plt.annotate(str(label[0]), xy=(label[1],label[2]),xytext=(label[1],label[2]), weight='bold',
                         color='b', fontsize=14)
    # add legend
    proxies = [Line2D([0, 1], [0, 1], color='orange', linewidth=0.5),
               Line2D([0, 1], [0, 1], color='orange', linewidth=5)]
    ax.legend(proxies, ['%s:%.4f' % ('volume', min(values)), '%s:%.4f' % ('volume', max(values))])
    # set axis
    ax.autoscale_view()
    plt.xlabel('x_coord')
    plt.ylabel('y_coord')
    plt.tight_layout()
    plt.show()
    # save fig
    if savefig:
        try:
            figname = savefig['filename'] if 'filename' in savefig.keys() else 'zone_demand_flow.png'
            dpi = savefig['dpi'] if 'dpi' in savefig else 300
            fig.savefig(figname, dpi=dpi, bbox_inches='tight')
        except Exception as e:
            print(e)

def plotNetByAgentTrace(node_coords,link_coords,poi_coords,trace_node_coords,trace_route_coords,savefig):
    fig, ax = plt.subplots(figsize=(10, 8))
    # plot network nodes
    ax.scatter(node_coords[0], node_coords[1], marker='o', c='red', s=10, zorder=1)
    # plot network links
    ax.add_collection(LineCollection(link_coords, colors='orange', linewidths=1, zorder=2))
    # plot network pois
    if len(poi_coords):
        coll = PolyCollection(poi_coords, alpha=0.7, zorder=0)
        ax.add_collection(coll)
    # plot trace node sequence
    ax.scatter(trace_node_coords[0],trace_node_coords[1], marker='o', c='black', s=15, zorder=2)
    # plot trace route
    ax.add_collection(LineCollection(trace_route_coords, colors='blue', linewidths=2, zorder=2))
    # set axis
    ax.autoscale_view()
    plt.xlabel('x_coord')
    plt.ylabel('y_coord')
    plt.tight_layout()
    # show fig
    plt.show()
    # save fig
    if savefig:
        try:
            figname = savefig['filename'] if 'filename' in savefig.keys() else 'agent_trace.png'
            dpi = savefig['dpi'] if 'dpi' in savefig else 300
            fig.savefig(figname, dpi=dpi, bbox_inches='tight')
        except Exception as e:
            print(e)




avl_node_attrs=['ctrl_type','activity_type','production','attraction']
avl_node_attrs_for_num=['ctrl_type','activity_type']
avl_node_attrs_for_range=['production','attraction']

avl_link_attrs=['link_type_name','allowed_uses','free_speed','lanes','capacity','length']
avl_link_attrs_for_num=['link_type_name','allowed_uses']
avl_link_attrs_for_range=['free_speed','lanes','capacity','length']

avl_poi_attrs=['building','activity_zone_id']


from .readfiles import *
from .plot import *
from .utils import *
from .setting import *

def get_num_of_nodes_by_attr(network, attr):
    """
    :param network:
    :param attr: str
    :return:
    """
    if attr not in avl_node_attrs_for_num:
        print("ValueError: {} is expected".format(avl_node_attrs_for_num))
    else:
        attr_values = statNodeAttrValue(network, attr)
        if len(attr_values)==0:
            print("the '%s' value not found in node.csv"%attr)
        else:
            print('%-20s%-20s' % (attr, 'number'))
            for k, v in attr_values.items():
                print('%-20s%-20s' % (k, v))

def get_range_of_node_attr(network,attr):
    """
    :param network:
    :param attr: str
    :return:
    """
    if attr not in avl_node_attrs_for_range:
        print("ValueError: {} is expected".format( avl_node_attrs_for_range))
    else:
        attr_values = statNodeAttrValue(network, attr)
        if len(attr_values) == 0:
            print("the '%s' value not found in node.csv" % attr)
        else:
            print('%-20s%-20s%-20s' % ('attr', 'min value', 'max value'))
            print('%-20s%-20.4f%-20.4f' % (attr, attr_values['min_v'],attr_values['max_v']))

def get_num_of_links_by_attr(network, attr):
    """
    :param network:
    :param attr: str
    :return:
    """
    if attr not in avl_link_attrs_for_num:
        print("ValueError: {} is expected".format(avl_link_attrs_for_num))
    else:
        attr_values = statLinkAttrValue(network, attr)
        if len(attr_values)==0:
            print("the '%s' value not found in link.csv"%attr)
        else:
            print('%-20s%-20s' % (attr, 'number'))
            for k, v in attr_values.items():
                print('%-20s%-20s' % (k, v))

def get_range_of_link_attr(network,attr):
    """
    :param network:
    :param attr: str
    :return:
    """
    if attr not in avl_link_attrs_for_range:
        print("ValueError: {} is expected".format(avl_link_attrs_for_range))
    else:
        attr_values = statLinkAttrValue(network, attr)
        if len(attr_values) == 0:
            print("the '%s' value not found in link.csv" % attr)
        else:
            print('%-20s%-20s%-20s' % ('attr', 'min value', 'max value'))
            print('%-20s%-20.4f%-20.4f' % (attr,attr_values['min_v'], attr_values['max_v']))

def get_num_of_pois_by_attr(network, attr):
    """
    :param network:
    :param attr: str
    :return:
    """
    if attr not in avl_poi_attrs:
        print("ValueError: {} is expected".format(avl_poi_attrs))
    else:
        attr_values = statPoiAttrValue(network, attr)
        if len(attr_values)==0:
            print("the '%s' attribute value missing in poi.csv"%attr)
        else:
            print('%-30s%-20s' % (attr, 'number'))
            for k, v in attr_values.items():
                print('%-30s%-20s' % (k, v))

def get_info_of_zone_by_id(network,zone_id=None):
    """
    :param network:
    :param zone_id: int
    :return:
    """
    print('%-20s%-20s%-20s%-20s%-20s'%('zone_id','name','total_poi_count','total_production','total_attraction'))
    if isinstance(zone_id,int):
        for k, zone in network.zone_dict.items():
            if zone.activity_zone_id==zone_id:
                print('%-20s%-20s%-20s%-20.4f%-20.4f' % (zone.activity_zone_id, zone.name,zone.total_poi_count,
                                                     zone.total_production, zone.total_attraction))
    else:
        for k, zone in network.zone_dict.items():
            print('%-20s%-20s%-20s%-20.4f%-20.4f' % (zone.activity_zone_id, zone.name, zone.total_poi_count,
                                                 zone.total_production, zone.total_attraction))


def readNetwork(input_folder=None):
    """
    :param input_folder: str (the file directory)
    :return:
    """
    if input_folder:
        node_filepath = os.path.join(input_folder, 'node.csv')
        link_filepath = os.path.join(input_folder, 'link.csv')
        poi_filepath = os.path.join(input_folder, 'poi.csv')
        demand_filepath=os.path.join(input_folder,'demand.csv')
        agent_filepath=os.path.join(input_folder,'input_agent.csv')
        poi_trip_filepath = os.path.join(input_folder, 'poi_trip_rate.csv')
        zone_filepath = os.path.join(input_folder, 'zone.csv')
    else:
        node_filepath = 'node.csv'
        link_filepath = 'link.csv'
        poi_filepath = 'poi.csv'
        demand_filepath = 'demand.csv'
        agent_filepath = 'input_agent.csv'
        poi_trip_filepath ='poi_trip_rate.csv'
        zone_filepath ='zone.csv'

    net=readNet(node_filepath,link_filepath,poi_filepath,zone_filepath,demand_filepath,poi_trip_filepath,agent_filepath)
    return net

def showNetByAllMode(network,savefig=None):
    """
    :param network:
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    plotNetbySelectedObj(network.node_coords,network.link_coords,network.poi_coords,savefig)

def showNetByAutoMode(network,savefig=None):
    """
    :param network:
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    auto_net=searchNetMode(network,['auto','all'])
    if len(auto_net[0][0])>0:
        plotNetbySelectedObj(auto_net[0],auto_net[1],network.poi_coords,savefig)
    else:
        print('error!  auto mode not found in the link.csv')

def showNetByWalkMode(network,savefig=None):
    """
    :param network:
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    walk_net = searchNetMode(network, ['walk','all'])
    if len(walk_net[0][0]) > 0:
        plotNetbySelectedObj(walk_net[0],walk_net[1], network.poi_coords,savefig)
    else:
        print('error! walk mode not found in the link.csv')

def showNetByBikeMode(network,savefig=None):
    """
    :param network:
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    bike_net = searchNetMode(network, ['bike','all'])
    if len(bike_net[0][0]) > 0:
        plotNetbySelectedObj(bike_net[0],bike_net[1], network.poi_coords,savefig)
    else:
        print('error! bike mode not found in the link.csv')

def showNetByRailMode(network,savefig=None):
    """
    :param network:
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    rail_net = searchNetMode(network, ['railway'])
    if len(rail_net[0][0]) > 0:
        plotNetbySelectedObj(rail_net[0],rail_net[1], network.poi_coords,savefig)
    else:
        print('error! rail mode not found in the link.csv')

def showNetByNodeAttr(network,attr_dict,savefig=None):
    """
    :param network:
    :param attr_dict: dict,for example: {'ctrl_type':1}
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    attr,value=list(attr_dict.items())[0]
    if attr not in avl_node_attrs:
        print("ValueError: ‘{}’ is not available node attributes, {} is expected".format(attr,avl_node_attrs))
    else:
        node_coords, link_coords=searchNetbyNodeAttr(network,attr,value)
        if len(node_coords[0]):
            plotNetbySelectedObj(node_coords,link_coords,network.poi_coords,savefig)
        else:
            print(" '{}={}' values not found".format(attr,value))
def showNetByNodeProduction(network,savefig=None):
    """
    :param network:
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    node_coords,values=statNodeAttrRange(network,'production')
    if len(values)>0:
        plotNetbyNodeAttrRange(node_coords, network.link_coords,network.poi_coords,attr='production',
                               value=values,savefig=savefig)
    else:
        print("node production values not found")
def showNetByNodeAttraction(network,savefig=None):
    """
    :param network:
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    node_coords, values = statNodeAttrRange(network,'attraction')
    if len(values)>0:
        plotNetbyNodeAttrRange(node_coords, network.link_coords, network.poi_coords, attr='attraction',
                               value=values,savefig=savefig)
    else:
        print("node attraction values not found")


def showNetByLinkAttr(network,attr_dict,savefig=None):
    """
    :param network:
    :param attr_dict: dict, for example: {'link_type_name':'secondary'}
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    attr, value = list(attr_dict.items())[0]
    if attr not in avl_link_attrs:
        print("ValueError: ‘{}’ is not available link attributes, {} is expected".format(attr,avl_link_attrs))
    else:
        node_coords, link_coords=searchNetbyLinkAttr(network,attr,value)
        if len(node_coords[0]):
            plotNetbySelectedObj(node_coords,link_coords,network.poi_coords,savefig)
        else:
            print(" '{}={}' values not found".format(attr,value))

def showNetByLinkFreeSpeed(network,savefig=None):
    """
    :param network:
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    node_coords,link_coords, values=statLinkAttrRange(network,'free_speed')
    if len(values)>0:
        plotNetbyLinkAttrRange(node_coords,link_coords,network.poi_coords,'free_speed',values,savefig)
    else:
        print("link free speed values not found")

def showNetByLinkLaneNum(network,savefig=None):
    """
    :param network:
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    node_coords,link_coords, values = statLinkAttrRange(network, 'lanes')
    if len(values)>0:
        plotNetbyLinkAttrRange(node_coords, link_coords, network.poi_coords, 'lanes', values,savefig)
    else:
        print("link lanes values not found")
def showNetByLinkCapacity(network,savefig=None):
    """
    :param network:
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    node_coords,link_coords, values = statLinkAttrRange(network, 'capacity')
    if len(values)>0:
        plotNetbyLinkAttrRange(node_coords, link_coords, network.poi_coords, 'capacity', values,savefig)
    else:
        print("link capacity values not found")

def showNetByPOIAttr(network,attr_dict,savefig=None):
    """
    :param network:
    :param attr_dict: dict, for example: {'building':'parking'}
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    attr, value = list(attr_dict.items())[0]
    if attr not in avl_poi_attrs:
        print("ValueError: ‘{}’ is not available poi attributes, {} is expected".format(attr,avl_poi_attrs))
    else:
        poi_coords=searchNetbyPoiAttr(network,attr,value)
        if len(poi_coords)>0:
            plotNetbySelectedObj(network.node_coords,network.link_coords,poi_coords,savefig)
        else:
            print(" '{}={}' values not found".format(attr,value))

def showNetByZonePOIProductionDensity(network,savefig=None):
    """
    :param network:
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    poi_coords, values=statPoiAttrRangeforDensity(network,attr='production_rate1')
    if len(poi_coords)>0:
        plotNetbyZonePoiAttrDensity(poi_coords,values,savefig)
    else:
        print("poi production values not found")
def showNetByZonePOIAttractionDensity(network,savefig=None):
    """
    :param network:
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    poi_coords, values=statPoiAttrRangeforDensity(network,attr='attraction_rate1')
    if len(poi_coords)>0:
        plotNetbyZonePoiAttrDensity(poi_coords,values,savefig)
    else:
        print("poi attraction values not found")

def showNetByZoneDemandHeatMap(network,annot=False,savefig=None):
    """
    :param network:
    :param annot: bool, whether show demand volume values
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    if network.number_of_zone==0:
        print("zone demand volume information not found, please check demand.csv")
    else:
        demand=statZoneDemandforHeat(network)
        plotNetbyZoneDeamndHeat(network,demand,annot,savefig)
def showNetByZoneDemandFlow(network,annot=True,bg=True,savefig=None):
    """
    :param network:
    :param annot: bool, whether show zone id
    :param bg: bool, whether show network as background
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    if network.number_of_demand==0:
        print("zone demand volume information not found, please check demand.csv")
    else:
        demand_line, values, zone_grid, zone_labels=statZoneDemandforFlow(network)
        plotNetbyZoneDemandFlow(network,demand_line,values, zone_grid, zone_labels,annot,bg,savefig)

def showNetByAgentTrace(network,agent_id,savefig=None):
    """
    :param network:
    :param agent_id: int
    :param savefig: dict for exampel:{'filename':'a.png','dpi':300}
    :return:
    """
    if agent_id not in network.agent_dict.keys():
        print(" 'agent id={}' not found".format(agent_id))
    else:
        agent_trace_node_coords, agent_trace_route_coords=statAgentTracebyID(network,agent_id)
        if len(agent_trace_node_coords[0])==0 and len(agent_trace_route_coords)==0:
            print("the trace of 'agent id={}' not found".format(agent_id))
        else:
            plotNetByAgentTrace(network.node_coords,network.link_coords,network.poi_coords,
                                agent_trace_node_coords,agent_trace_route_coords,savefig)





import chardet
import numpy as np
from shapely import geometry

def get_encoding(file):
    with open(file,'rb') as f:
        tmp = chardet.detect(f.read())
        return tmp['encoding']

def statNodeAttrValue(network,attr):
    attr_values={}
    try:
        if attr == 'ctrl_type':
            for k, node in network.node_dict.items():
                if isinstance(node.ctrl_type,int):
                    if node.ctrl_type not in attr_values.keys() and isinstance(node.ctrl_type,int):
                        attr_values[node.ctrl_type]=1
                    else:
                        attr_values[node.ctrl_type]+=1
        elif attr == 'activity_type':
            for k, node in network.node_dict.items():
                if node.activity_type:
                    if node.activity_type not in attr_values.keys() and node.activity_type:
                        attr_values[node.activity_type]=1
                    else:
                        attr_values[node.activity_type]+=1
        elif attr == 'production':
            min_v=float('inf')
            max_v=-float('inf')
            for k, node in network.node_dict.items():
                if isinstance(node.production,float):
                    if node.production<min_v:
                        min_v=node.production
                    if node.production>max_v:
                        max_v=node.production
            attr_values = {'min_v': min_v, 'max_v': max_v}
        elif attr == 'attraction':
            min_v=float('inf')
            max_v=-float('inf')
            for k, node in network.node_dict.items():
                if isinstance(node.attraction,float):
                    if node.attraction<min_v:
                        min_v=node.attraction
                    if node.attraction>max_v:
                        max_v=node.attraction
            attr_values={'min_v':min_v,'max_v':max_v}
    except Exception as e:
        print(e)
    return attr_values

def statLinkAttrValue(network,attr):
    attr_values={}
    try:
        if attr == 'length':
            min_v=float('inf')
            max_v=-float('inf')
            for k, link in network.link_dict.items():
                if isinstance(link.length,float):
                    if link.length>max_v:
                        max_v=link.length
                    if link.length<min_v:
                        min_v=link.length
            attr_values={'min_v':min_v,'max_v':max_v}
        elif attr == 'lanes':
            min_v = float('inf')
            max_v = -float('inf')
            for k, link in network.link_dict.items():
                if isinstance(link.lanes,int):
                    if link.lanes>max_v:
                        max_v=link.lanes
                    if link.lanes<min_v:
                        min_v=link.lanes
            attr_values = {'min_v': min_v, 'max_v': max_v}
        elif attr == 'free_speed':
            min_v = float('inf')
            max_v = -float('inf')
            for k, link in network.link_dict.items():
                if isinstance(link.free_speed,float):
                    if link.free_speed>max_v:
                        max_v=link.free_speed
                    if link.free_speed<min_v:
                        min_v=link.free_speed
            attr_values = {'min_v': min_v, 'max_v': max_v}
        elif attr == 'capacity':
            min_v = float('inf')
            max_v = -float('inf')
            for k, link in network.link_dict.items():
                if isinstance(link.capacity,float):
                    if link.capacity>max_v:
                        max_v=link.capacity
                    if link.capacity<min_v:
                        min_v=link.capacity
            attr_values = {'min_v': min_v, 'max_v': max_v}
        elif attr == 'link_type_name':
            for k, link in network.link_dict.items():
                if isinstance(link.link_type_name,str):
                    if link.link_type_name not in attr_values.keys() :
                        attr_values[link.link_type_name] = 1
                    else:
                        attr_values[link.link_type_name] += 1
        elif attr == 'allowed_uses':
            for k, link in network.link_dict.items():
                for allowed_uses in link.allowed_uses:
                    if allowed_uses not in attr_values:
                        attr_values[allowed_uses] = 1
                    else:
                        attr_values[allowed_uses] += 1
    except Exception as e:
        print(e)
    return attr_values

def statPoiAttrValue(network,attr):
    attr_values={}
    try:
        if attr == 'building':
            for k, poi in network.poi_dict.items():
                for building in poi.building:
                    if building not in attr_values.keys() and isinstance(building,str):
                        attr_values[building] = 1
                    else:
                        attr_values[building] += 1
        elif attr == 'activity_zone_id':
            for k, poi in network.poi_dict.items():
                if isinstance(poi.activity_zone_id,int):
                    if poi.activity_zone_id not in attr_values.keys() and isinstance(poi.activity_zone_id,int):
                        attr_values[poi.activity_zone_id] = 1
                    else:
                        attr_values[poi.activity_zone_id] += 1
    except Exception as e:
        print(e)
    return attr_values

def searchNetMode(network,net_type):
    node_x_coords=[]
    node_y_coords=[]
    link_coords=[]
    unique_links=[]
    for k,link in network.link_dict.items():
        if (link.from_node_id, link.to_node_id) not in unique_links and (link.to_node_id, link.from_node_id) not in unique_links:
            coords = list(link.geometry.coords)
            for allowed_uses in link.allowed_uses:
                if allowed_uses in net_type:
                    try:
                        from_node = network.node_dict[link.from_node_id]
                        to_node = network.node_dict[link.to_node_id]
                        node_x_coords.extend([from_node.x_coord,to_node.x_coord])
                        node_y_coords.extend([from_node.y_coord,to_node.y_coord])
                    except:
                        pass
                    link_coords.append(np.array(coords))
                    unique_links.append((link.from_node_id, link.to_node_id))
    node_coords=[node_x_coords,node_y_coords]
    return [node_coords,link_coords]

def searchNetbyNodeAttr(network,attr,value):
    node_x_coords = []
    node_y_coords = []
    link_coords = []
    unique_links = []
    try:
        if attr == 'ctrl_type':
            if isinstance(value, int):
                for k, node in network.node_dict.items():
                    if node.ctrl_type ==value:
                        node_x_coords.append(node.x_coord)
                        node_y_coords.append(node.y_coord)
                        for out_link in node.out_link_list:
                            if (node.node_id, out_link.to_node_id) not in unique_links and (out_link.to_node_id, node.node_id) not in unique_links:
                                coords = list(out_link.geometry.coords)
                                link_coords.append(np.array(coords))
                                unique_links.append((node.node_id, out_link.to_node_id))
                        for in_link in node.in_link_list:
                            if (in_link.from_node_id, node.node_id) not in unique_links and \
                                    (node.node_id, in_link.from_node_id) not in unique_links:
                                coords = list(in_link.geometry.coords)
                                link_coords.append(np.array(coords))
                                unique_links.append((in_link.from_node_id, node.node_id))
            elif isinstance(value, tuple):
                if value[0]>value[1]:
                    print('please input available ctrl_type range')
                else:
                    for k, node in network.node_dict.items():
                        if isinstance(node.ctrl_type,int):
                            if node.ctrl_type <=value[1] and node.ctrl_type>=value[0]:
                                node_x_coords.append(node.x_coord)
                                node_y_coords.append(node.y_coord)
                                for out_link in node.out_link_list:
                                    if (node.node_id, out_link.to_node_id) not in unique_links and (
                                    out_link.to_node_id, node.node_id) not in unique_links:
                                        coords = list(out_link.geometry.coords)
                                        link_coords.append(np.array(coords))
                                    unique_links.append((node.node_id, out_link.to_node_id))
                                for in_link in node.in_link_list:
                                    if (in_link.from_node_id, node.node_id) not in unique_links and \
                                            (node.node_id, in_link.from_node_id) not in unique_links:
                                        coords = list(in_link.geometry.coords)
                                        link_coords.append(np.array(coords))
                                    unique_links.append((in_link.from_node_id, node.node_id))
        elif attr == 'activity_type':
            if isinstance(value,str):
                for k, node in network.node_dict.items():
                    if node.activity_type == value:
                        node_x_coords.append(node.x_coord)
                        node_y_coords.append(node.y_coord)
                        for out_link in node.out_link_list:
                            if (node.node_id, out_link.to_node_id) not in unique_links and (
                                    out_link.to_node_id, node.node_id) not in unique_links:
                                coords = list(out_link.geometry.coords)
                                link_coords.append(np.array(coords))
                                unique_links.append((node.node_id, out_link.to_node_id))
                        for in_link in node.in_link_list:
                            if (in_link.from_node_id, node.node_id) not in unique_links and \
                                    (node.node_id, in_link.from_node_id) not in unique_links:
                                coords = list(in_link.geometry.coords)
                                link_coords.append(np.array(coords))
                                unique_links.append((in_link.from_node_id, node.node_id))
            elif isinstance(value,list):
                for k, node in network.node_dict.items():
                    if node.activity_type in value:
                        node_x_coords.append(node.x_coord)
                        node_y_coords.append(node.y_coord)
                        for out_link in node.out_link_list:
                            if (node.node_id, out_link.to_node_id) not in unique_links and (
                                    out_link.to_node_id, node.node_id) not in unique_links:
                                coords = list(out_link.geometry.coords)
                                link_coords.append(np.array(coords))
                            unique_links.append((node.node_id, out_link.to_node_id))
                        for in_link in node.in_link_list:
                            if (in_link.from_node_id, node.node_id) not in unique_links and \
                                    (node.node_id, in_link.from_node_id) not in unique_links:
                                coords = list(in_link.geometry.coords)
                                link_coords.append(np.array(coords))
                            unique_links.append((in_link.from_node_id, node.node_id))
        elif attr=='production':
            if isinstance(value,tuple):
                if value[0] > value[1]:
                    print('please input available node production range')
                else:
                    for k, node in network.node_dict.items():
                        if isinstance(node.production,float):
                            if node.production <= value[1] and node.production >= value[0]:
                                node_x_coords.append(node.x_coord)
                                node_y_coords.append(node.y_coord)
                                for out_link in node.out_link_list:
                                    if (node.node_id, out_link.to_node_id) not in unique_links and (
                                            out_link.to_node_id, node.node_id) not in unique_links:
                                        coords = list(out_link.geometry.coords)
                                        link_coords.append(np.array(coords))
                                        unique_links.append((node.node_id, out_link.to_node_id))
                                for in_link in node.in_link_list:
                                    if (in_link.from_node_id, node.node_id) not in unique_links and \
                                            (node.node_id, in_link.from_node_id) not in unique_links:
                                        coords = list(in_link.geometry.coords)
                                        link_coords.append(np.array(coords))
                                        unique_links.append((in_link.from_node_id, node.node_id))
        elif attr=='attraction':
            if isinstance(value, tuple):
                if value[0] > value[1]:
                    print('please input available node attraction range')
                else:
                    for k, node in network.node_dict.items():
                        if isinstance(node.attraction,float):
                            if node.attraction <= value[1] and node.attraction >= value[0]:
                                node_x_coords.append(node.x_coord)
                                node_y_coords.append(node.y_coord)
                                for out_link in node.out_link_list:
                                    if (node.node_id, out_link.to_node_id) not in unique_links and (
                                            out_link.to_node_id, node.node_id) not in unique_links:
                                        coords = list(out_link.geometry.coords)
                                        link_coords.append(np.array(coords))
                                        unique_links.append((node.node_id, out_link.to_node_id))
                                for in_link in node.in_link_list:
                                    if (in_link.from_node_id, node.node_id) not in unique_links and \
                                            (node.node_id, in_link.from_node_id) not in unique_links:
                                        coords = list(in_link.geometry.coords)
                                        link_coords.append(np.array(coords))
                                        unique_links.append((in_link.from_node_id, node.node_id))
    except Exception as e:
        print(e)
    node_coords=[node_x_coords,node_y_coords]
    return [node_coords,link_coords]

def statNodeAttrRange(network,attr):
    node_x_coords = []
    node_y_coords = []
    values=[]
    try:
        if attr=='production' :
            for k,node in network.node_dict.items():
                if isinstance(node.production,float):
                    node_x_coords.append(node.x_coord)
                    node_y_coords.append(node.y_coord)
                    values.append(node.production)
        elif attr=='attraction':
            for k, node in network.node_dict.items():
                if isinstance(node.attraction,float):
                    node_x_coords.append(node.x_coord)
                    node_y_coords.append(node.y_coord)
                    values.append(node.attraction)
    except Exception as e:
        print(e)
    node_coords=[node_x_coords,node_y_coords]
    return node_coords,values

def searchNetbyLinkAttr(network,attr,value):
    node_x_coords = []
    node_y_coords = []
    link_coords = []
    unique_links = []
    try:
        if attr=='length':
            if isinstance(value,tuple):
                for k, link in network.link_dict.items():
                    if isinstance(link.length, float):
                        from_node = network.node_dict[link.from_node_id]
                        to_node = network.node_dict[link.to_node_id]
                        if (link.from_node_id, link.to_node_id) not in unique_links and (link.to_node_id, link.from_node_id) not in unique_links:
                            if link.length <= value[1] and link.length >= value[0]:
                                coords = list(link.geometry.coords)
                                node_x_coords.extend([from_node.x_coord, to_node.x_coord])
                                node_y_coords.extend([from_node.y_coord, to_node.y_coord])
                                link_coords.append(np.array(coords))
                                unique_links.append((link.from_node_id, link.to_node_id))
        elif attr=='lanes':
            if isinstance(value, tuple):
                for k, link in network.link_dict.items():
                    if isinstance(link.lanes, int):
                        from_node = network.node_dict[link.from_node_id]
                        to_node = network.node_dict[link.to_node_id]
                        if (link.from_node_id, link.to_node_id) not in unique_links and (link.to_node_id, link.from_node_id) not in unique_links:
                            if link.lanes <= value[1] and link.lanes >= value[0]:
                                coords = list(link.geometry.coords)
                                node_x_coords.extend([from_node.x_coord, to_node.x_coord])
                                node_y_coords.extend([from_node.y_coord, to_node.y_coord])
                                link_coords.append(np.array(coords))
                                unique_links.append((link.from_node_id, link.to_node_id))
        elif attr=='free_speed':
            if isinstance(value, tuple):
                for k, link in network.link_dict.items():
                    if isinstance(link.free_speed,float):
                        from_node = network.node_dict[link.from_node_id]
                        to_node = network.node_dict[link.to_node_id]
                        if (link.from_node_id, link.to_node_id) not in unique_links and (
                                link.to_node_id, link.from_node_id) not in unique_links:
                            if link.free_speed <= value[1] and link.free_speed >= value[0]:
                                coords = list(link.geometry.coords)
                                node_x_coords.extend([from_node.x_coord, to_node.x_coord])
                                node_y_coords.extend([from_node.y_coord, to_node.y_coord])
                                link_coords.append(np.array(coords))
                                unique_links.append((link.from_node_id, link.to_node_id))
        elif attr=='capacity':
            if isinstance(value, tuple):
                for k, link in network.link_dict.items():
                    if isinstance(link.capacity,float):
                        from_node = network.node_dict[link.from_node_id]
                        to_node = network.node_dict[link.to_node_id]
                        if (link.from_node_id, link.to_node_id) not in unique_links and (
                                link.to_node_id, link.from_node_id) not in unique_links:
                            if link.capacity <= value[1] and link.capacity >= value[0]:
                                coords = list(link.geometry.coords)
                                node_x_coords.extend([from_node.x_coord, to_node.x_coord])
                                node_y_coords.extend([from_node.y_coord, to_node.y_coord])
                                link_coords.append(np.array(coords))
                                unique_links.append((link.from_node_id, link.to_node_id))
        elif attr=='link_type_name':
            if isinstance(value,str):
                for k, link in network.link_dict.items():
                    from_node = network.node_dict[link.from_node_id]
                    to_node = network.node_dict[link.to_node_id]
                    if (link.from_node_id, link.to_node_id) not in unique_links and (
                            link.to_node_id, link.from_node_id) not in unique_links:
                        if link.link_type_name==value:
                            coords = list(link.geometry.coords)
                            node_x_coords.extend([from_node.x_coord, to_node.x_coord])
                            node_y_coords.extend([from_node.y_coord, to_node.y_coord])
                            link_coords.append(np.array(coords))
                            unique_links.append((link.from_node_id, link.to_node_id))
            elif isinstance(value,list):
                for k, link in network.link_dict.items():
                    from_node = network.node_dict[link.from_node_id]
                    to_node = network.node_dict[link.to_node_id]
                    if (link.from_node_id, link.to_node_id) not in unique_links and (
                            link.to_node_id, link.from_node_id) not in unique_links:
                        if link.link_type_name in value:
                            coords = list(link.geometry.coords)
                            node_x_coords.extend([from_node.x_coord, to_node.x_coord])
                            node_y_coords.extend([from_node.y_coord, to_node.y_coord])
                            link_coords.append(np.array(coords))
                            unique_links.append((link.from_node_id, link.to_node_id))
        elif attr=='allowed_uses':
            if isinstance(value, str):
                for k, link in network.link_dict.items():
                    if (link.from_node_id, link.to_node_id) not in unique_links and (
                            link.to_node_id, link.from_node_id) not in unique_links:
                        for allowed_uses in link.allowed_uses:
                            if allowed_uses == value:
                                coords = list(link.geometry.coords)
                                try:
                                    from_node = network.node_dict[link.from_node_id]
                                    to_node = network.node_dict[link.to_node_id]
                                    node_x_coords.extend([from_node.x_coord, to_node.x_coord])
                                    node_y_coords.extend([from_node.y_coord, to_node.y_coord])
                                except:
                                    pass
                                link_coords.append(np.array(coords))
                                unique_links.append((link.from_node_id, link.to_node_id))
            elif isinstance(value, list):
                for k, link in network.link_dict.items():

                    if (link.from_node_id, link.to_node_id) not in unique_links and (
                            link.to_node_id, link.from_node_id) not in unique_links:
                        for allowed_uses in link.allowed_uses:
                            if allowed_uses in value:
                                coords = list(link.geometry.coords)
                                try:
                                    from_node = network.node_dict[link.from_node_id]
                                    to_node = network.node_dict[link.to_node_id]
                                    node_x_coords.extend([from_node.x_coord, to_node.x_coord])
                                    node_y_coords.extend([from_node.y_coord, to_node.y_coord])
                                except:
                                    pass
                                link_coords.append(np.array(coords))
                                unique_links.append((link.from_node_id, link.to_node_id))
    except Exception as e:
        print(e)
    node_coords = [node_x_coords, node_y_coords]
    return node_coords,link_coords

def statLinkAttrRange(network,attr):
    node_x_coords=[]
    node_y_coords=[]
    link_coords=[]
    unique_links = []
    values=[]
    try:
        if attr=='lanes':
            for k,link in network.link_dict.items():
                if isinstance(link.lanes, int):
                    if (link.from_node_id, link.to_node_id) not in unique_links and (link.to_node_id, link.from_node_id) not in unique_links:
                        coords = list(link.geometry.coords)
                        link_coords.append(np.array(coords))
                        unique_links.append((link.from_node_id, link.to_node_id))
                        values.append(link.lanes)
                        from_node = network.node_dict[link.from_node_id]
                        to_node = network.node_dict[link.to_node_id]
                        node_x_coords.extend([from_node.x_coord, to_node.x_coord])
                        node_y_coords.extend([from_node.y_coord, to_node.y_coord])
        elif attr=='free_speed':
            for k, link in network.link_dict.items():
                if isinstance(link.free_speed, float):
                    if (link.from_node_id, link.to_node_id) not in unique_links and (link.to_node_id, link.from_node_id) not in unique_links:
                        coords = list(link.geometry.coords)
                        link_coords.append(np.array(coords))
                        unique_links.append((link.from_node_id, link.to_node_id))
                        values.append(link.free_speed)
                        from_node = network.node_dict[link.from_node_id]
                        to_node = network.node_dict[link.to_node_id]
                        node_x_coords.extend([from_node.x_coord, to_node.x_coord])
                        node_y_coords.extend([from_node.y_coord, to_node.y_coord])
        else:
            for k, link in network.link_dict.items():
                if isinstance(link.capacity, float):
                    if (link.from_node_id, link.to_node_id) not in unique_links and (
                    link.to_node_id, link.from_node_id) not in unique_links:
                        coords = list(link.geometry.coords)
                        link_coords.append(np.array(coords))
                        unique_links.append((link.from_node_id, link.to_node_id))
                        values.append(link.capacity)
                        from_node = network.node_dict[link.from_node_id]
                        to_node = network.node_dict[link.to_node_id]
                        node_x_coords.extend([from_node.x_coord, to_node.x_coord])
                        node_y_coords.extend([from_node.y_coord, to_node.y_coord])
    except Exception as e:
        print(e)
    node_coords=[node_x_coords,node_y_coords]
    return node_coords,link_coords,values

def searchNetbyPoiAttr(network,attr,value):
    poi_coords=[]
    try:
        if attr=='building':
            if isinstance(value,str):
                for k,poi in network.poi_dict.items():
                    for building in poi.building:
                        if building==value:
                            if isinstance(poi.geometry, geometry.MultiPolygon):
                                for geom in poi.geometry.geoms:
                                    coords = list(geom.exterior.coords)
                                    poi_coords.append(np.array(coords))
                            elif isinstance(poi.geometry, geometry.Polygon):
                                coords = list(poi.geometry.exterior.coords)
                                poi_coords.append(np.array(coords))
            elif isinstance(value,list):
                for k, poi in network.poi_dict.items():
                    for building in poi.building:
                        if building in value:
                            if isinstance(poi.geometry, geometry.MultiPolygon):
                                for geom in poi.geometry.geoms:
                                    coords = list(geom.exterior.coords)
                                    poi_coords.append(np.array(coords))
                            elif isinstance(poi.geometry, geometry.Polygon):
                                coords = list(poi.geometry.exterior.coords)
                                poi_coords.append(np.array(coords))
        elif attr=='activity_zone_id':
            if isinstance(value,tuple):
                for k, poi in network.poi_dict.items():
                    if poi.activity_zone_id <= value[1] and poi.activity_zone_id>=value[0]:
                        if isinstance(poi.geometry, geometry.MultiPolygon):
                            for geom in poi.geometry.geoms:
                                coords = list(geom.exterior.coords)
                                poi_coords.append(np.array(coords))
                        elif isinstance(poi.geometry, geometry.Polygon):
                            coords = list(poi.geometry.exterior.coords)
                            poi_coords.append(np.array(coords))
    except Exception as e:
        print(e)
    return poi_coords

def statPoiAttrRangeforDensity(network,attr):
    poi_coords=[]
    values=[]
    try:
        if attr=='production_rate1':
            for k,poi in network.poi_dict.items():
                if isinstance(poi.geometry, geometry.MultiPolygon):
                    for geom in poi.geometry.geoms:
                        coords = list(geom.exterior.coords)
                        poi_coords.append(np.array(coords))
                        v=0
                        for building in poi.building:
                           v=v+ network.poi_trip_dict[building].production_rate1
                        values.append(v)
                elif isinstance(poi.geometry, geometry.Polygon):
                    coords = list(poi.geometry.exterior.coords)
                    poi_coords.append(np.array(coords))
                    v = 0
                    for building in poi.building:
                        v = v + network.poi_trip_dict[building].production_rate1
                    values.append(v)
        elif attr=='attraction_rate1':
            for k, poi in network.poi_dict.items():
                if isinstance(poi.geometry, geometry.MultiPolygon):
                    for geom in poi.geometry.geoms:
                        coords = list(geom.exterior.coords)
                        poi_coords.append(np.array(coords))
                        v = 0
                        for building in poi.building:
                            v = v + network.poi_trip_dict[building].attraction_rate1
                        values.append(v)
                elif isinstance(poi.geometry, geometry.Polygon):
                    coords = list(poi.geometry.exterior.coords)
                    poi_coords.append(np.array(coords))
                    v = 0
                    for building in poi.building:
                        v = v + network.poi_trip_dict[building].attraction_rate1
                    values.append(v)
    except Exception as e:
        print(e)
    return poi_coords,values

def statZoneDemandforHeat(network):
    data=np.zeros((network.number_of_zone,network.number_of_zone))
    try:
        for k,demand in network.demand_dict.items():
            o=demand.o_zone_id
            d=demand.d_zone_id
            data[o-1,d-1]=demand.volume
    except Exception as e:
        print(e)
    return data
def statZoneDemandforFlow(network):
    demand_line=[]
    values=[]
    zone_grid=[]
    zone_labels=[]
    try:
        # statistics demand flow
        for k,demand in network.demand_dict.items():
            if isinstance(demand.geometry,geometry.LineString):
                if demand.volume>0:
                    coords = list(demand.geometry.coords)
                    demand_line.append(np.array(coords))
                    values.append(demand.volume)
        for k,zone in network.zone_dict.items():
            if isinstance(zone.geometry,geometry.Polygon):
                coords = list(zone.geometry.exterior.coords)
                zone_grid.append(np.array(coords))
            zone_labels.append([zone.activity_zone_id,zone.centroid_x,zone.centroid_y])
    except Exception as e:
        print(e)
    return demand_line,values,zone_grid,zone_labels

def statAgentTracebyID(network,agent_id):
    agent_trace_route_coords = []
    node_x_coords=[]
    node_y_coords=[]
    agent=network.agent_dict[agent_id]
    if isinstance(agent.geometry, geometry.LineString):
        coords = list(agent.geometry.coords)
        agent_trace_route_coords.append(np.array(coords))
    for node_id in agent.node_sequence:
        try:
            node_x_coords.append(network.node_dict[node_id].x_coord)
            node_y_coords.append(network.node_dict[node_id].y_coord)
        except:
            pass
    agent_trace_node_coords=[node_x_coords,node_y_coords]
    return agent_trace_node_coords,agent_trace_route_coords



import sys
from shapely.wkt import loads
from shapely import geometry
class Node():
    def __init__(self,kwargs,row):
        keys=kwargs.keys()

        node_id=kwargs['node_id'] if 'node_id' in keys else ''
        if node_id:
            try:
                self.node_id=int(float(node_id))
            except Exception as e:
                print("broken at row{},".format(row),end=' ')
                print(e)
                sys.exit(0)
        else:
            print("node_id is not defined in node.csv, please check it!")
            sys.exit(0)

        ctrl_type=kwargs['ctrl_type'] if 'ctrl_type' in keys else ''
        try:
            self.ctrl_type=int(float(ctrl_type))
        except:
            self.ctrl_type=0
        self.activity_type = kwargs['activity_type'] if 'activity_type' in keys else 'unclassified'

        x_coord=kwargs['x_coord'] if 'x_coord' in keys else ''
        if x_coord:
            try:
                self.x_coord=float(x_coord)
            except Exception as e:
                print('broken at row {},'.format(row),end=' ')
                print(e)
                sys.exit(0)
        else:
            print("x_coord not found in node.csv, please check it!")
            sys.exit(0)
        y_coord=kwargs['y_coord'] if 'y_coord' in keys else ''
        if y_coord:
            try:
                self.y_coord=float(y_coord)
            except Exception as e:
                print('broken at row {},'.format(row), end=' ')
                print(e)
                sys.exit(0)
        else:
            print("y_coord not found in node.csv, please check it!")
            sys.exit(0)

        self.geometry=geometry.Point(self.x_coord,self.y_coord)
        production=kwargs['production'] if 'production' in keys else ''
        try:
            self.production=float(production)
        except:
            self.production=''
        attraction=kwargs['attraction'] if 'attraction' in keys else ''
        try:
            self.attraction=float(attraction)
        except:
            self.attraction=''

        self.out_link_list=[]
        self.in_link_list=[]


class Link():
    def __init__(self,kwargs,row):
        keys=kwargs.keys()

        link_id=kwargs['link_id'] if 'link_id' in keys else ''
        if link_id:
            try:
                self.link_id=int(float(link_id))
            except Exception as e:
                print('broken at row {},'.format(row),end=' ')
                print(e)
                sys.exit(0)
        else:
            self.link_id=None
        from_node_id=kwargs['from_node_id'] if 'from_node_id' in keys else ''
        if from_node_id:
            try:
                self.from_node_id=int(float(from_node_id))
            except Exception as e:
                print('broken at row {},'.format(row), end=' ')
                print(e)
                sys.exit(0)
        else:
            print("from_node_id not found in link.csv, please check it!")
            sys.exit(0)
        to_node_id=kwargs['to_node_id'] if 'to_node_id' in keys else ''
        if to_node_id:
            try:
                self.to_node_id=int(float(to_node_id))
            except Exception as e:
                print('broken at row {},'.format(row), end=' ')
                print(e)
                sys.exit(0)
        else:
            print("to_node_id not found in link.csv, please check it!")
            sys.exit(0)
        length=kwargs['length'] if 'length' in keys else ''
        try:
            self.length =float(length)
        except:
            self.length=''
        lanes=kwargs['lanes'] if 'lanes' in keys else ''
        try:
            self.lanes =int(float(lanes))
        except:
            self.lanes=''

        free_speed=kwargs['free_speed'] if 'free_speed' in keys else ''
        try:
            self.free_speed =float(free_speed)
        except:
            self.free_speed=''
        capacity=kwargs['capacity'] if 'capacity' in keys else ''
        try:
            self.capacity = float(capacity)
        except:
            self.capacity=''
        link_type_name=kwargs['link_type_name'] if 'link_type_name' in keys else ''
        if link_type_name:
            self.link_type_name=link_type_name
        else:
            self.link_type_name='unclassified'
        link_geo=kwargs['geometry'] if 'geometry' in keys else ''
        try:
            self.geometry = loads(link_geo)
        except:
            self.geometry=''
        if 'allowed_uses' in keys:
            allowed_uses=kwargs['allowed_uses']
            if allowed_uses:
                if ',' in allowed_uses:
                    self.allowed_uses=[allowed_use_.lstrip() for allowed_use_ in allowed_uses.split(',')]
                elif ';' in allowed_uses:
                    self.allowed_uses = [allowed_use_.lstrip() for allowed_use_ in allowed_uses.split(';')]
                else:
                    self.allowed_uses = [allowed_uses]
            else:
                self.allowed_uses = ['unclassified']
        else:
            self.allowed_uses=['unclassified']



class Agent():
    def __init__(self,kwargs,row):
        keys=kwargs.keys()
        agent_id=kwargs['agent_id'] if 'agent_id' in keys else ''
        if agent_id:
            self.agent_id=int(float(agent_id))
        else:
            self.agent_id=None
        node_sequence=kwargs['node_sequence'] if 'node_sequence' in keys else ''
        try:
            self.node_sequence=[int(float(id)) for id in node_sequence.split(';')[:-1]]
        except:
            self.node_sequence=''
        agent_geo=kwargs['geometry'] if 'geometry' in keys else ''
        if ',)' in agent_geo:
            agent_geo=agent_geo.replace(',)',')')
        try:
            self.geometry=loads(agent_geo)
        except:
            self.geometry=''
            print("warning: can't load geometry at row{}".format(row))


class Demand():
    def __init__(self,kwargs,row):
        keys=kwargs.keys()
        o_zone_id=kwargs['o_zone_id'] if 'o_zone_id' in keys else ''
        if o_zone_id:
            try:
                self.o_zone_id=int(float(o_zone_id))
            except Exception as e:
                print('broken at row {},'.format(row),end=' ')
                print(e)
                sys.exit(0)
        else:
            print("o_zone_id is not defined in demand.csv, please check it!")
            sys.exit(0)
        d_zone_id=kwargs['d_zone_id'] if 'd_zone_id' in keys else ''
        if d_zone_id:
            try:
                self.d_zone_id=int(float(d_zone_id))
            except Exception as e:
                print('broken at row {},'.format(row),end=' ')
                print(e)
                sys.exit(0)
        else:
            print("d_zone_id is not defined in demand.csv, please check it!")
            sys.exit(0)
        vol=kwargs['volume'] if 'volume' in keys else ''
        if vol:
            try:
                self.volume=float(vol)
            except Exception as e:
                print('broken at row {},'.format(row),end=' ')
                print(e)
                sys.exit(0)
        else:
            print("volume is not defined in demand.csv, please check it!")
            sys.exit(0)
        demand_geo=kwargs['geometry'] if 'geometry' in keys else ''
        if demand_geo:
            try:
                self.geometry=loads(demand_geo)
            except Exception as e:
                print('broken at row {},'.format(row),end=' ')
                print(e)
                sys.exit(0)
        else:
            print("geometry is not defined in demand.csv, please check it!")
            sys.exit(0)

class POI():
    def __init__(self,kwargs,row):
        keys=kwargs.keys()
        poi_id=kwargs['poi_id'] if 'poi_id' in keys else ''
        try:
            self.poi_id=int(float(poi_id))
        except :
            self.poi_id=None
        self.name=kwargs['name'] if 'name' in keys else ''
        building=kwargs['building'] if 'building' in keys else ''
        if building:
            self.building=building.split(';')
        else:
            self.building = ['unclassified']
        poi_geo=kwargs['geometry'] if 'geometry' in keys else ''
        if poi_geo:
            try:
                self.geometry=loads(poi_geo)
            except Exception as e:
                print('broken at row {}'.format(row),end=' ')
                print(e)
                sys.exit(0)
        else:
            print("geometry is not defined in poi.csv, please check it!")
            sys.exit(0)
        centroid=kwargs['centroid'] if 'centroid' in keys else ''
        if centroid:
            try:
                self.centroid=loads(centroid)
            except:
                self.centroid = self.geometry.centroid
        else:
            self.centroid=self.geometry.centroid

        activity_zone_id=kwargs['activity_zone_id'] if 'activity_zone_id' in keys else ''
        try:
            self.activity_zone_id=int(float(activity_zone_id))
        except:
            self.activity_zone_id=''

class POITrip():
    def __init__(self,kwargs,row):
        keys=kwargs.keys()
        building = kwargs['building'] if 'building' in keys else ''
        if building:
            self.building = building.split(';')
        else:
            self.building =['unclassified']
        production_rate1=kwargs['production_rate1'] if 'production_rate1' in keys else ''
        if production_rate1:
            try:
                self.production_rate1=float(production_rate1)
            except:
                self.production_rate1=0
        else:
            print("production_rate1 is not defined in poi_trip_rate.csv, please check it!")
            sys.exit(0)
        attraction_rate1=kwargs['attraction_rate1'] if 'attraction_rate1' in keys else ''
        if attraction_rate1:
            try:
                self.attraction_rate1=float(attraction_rate1)
            except:
                self.attraction_rate1=0
        else:
            print("attraction_rate1 is not defined in poi_trip_rate.csv, please check it!")
            sys.exit(0)


class Zone():
    def __init__(self,kwargs,row):
        keys=kwargs.keys()
        self.name=kwargs['name'] if 'name' in keys else ' '
        activity_zone_id=kwargs['activity_zone_id'] if 'activity_zone_id' in keys else ''
        if activity_zone_id:
            try:
                self.activity_zone_id=int(float(activity_zone_id))
            except Exception as e:
                print("broken at row{}".format(row),end=' ')
                print(e)
                sys.exit(0)
        else:
            print("activity_zone_id is not defined in zone.csv, please check it!")
            sys.exit(0)
        centroid_x=kwargs['centroid_x'] if 'centroid_x' in keys else ''
        if centroid_x:
            try:
                self.centroid_x=float(centroid_x)
            except Exception as e:
                print("broken at row{}".format(row),end=' ')
                print(e)
                sys.exit(0)
        else:
            print("centroid_x is not defined in zone.csv, please check it!")
            sys.exit(0)
        centroid_y=kwargs['centroid_y'] if 'centroid_y' in keys else ''
        if centroid_y:
            try:
                self.centroid_y=float(centroid_y)
            except Exception as e:
                print("broken at row{}".format(row), end=' ')
                print(e)
                sys.exit(0)
        else:
            print("centroid_y is not defined in zone.csv, please check it!")
            sys.exit(0)
        zone_geo=kwargs['geometry'] if 'geometry' in keys else ''
        if zone_geo:
            try:
                self.geometry=loads(zone_geo)
            except Exception as e:
                print("broken at row{}".format(row), end=' ')
                print(e)
                sys.exit(0)
        else:
            self.geometry=''
        centroid=kwargs['centroid'] if 'centroid' in keys else ''
        if centroid:
            try:
                self.centroid=loads(centroid)
            except Exception as e:
                print("broken at row{}".format(row), end=' ')
                print(e)
                sys.exit(0)
        else:
            print("centroid is not defined in zone.csv, please check it!")
            sys.exit(0)
        total_poi_count=kwargs['total_poi_count'] if 'total_poi_count' in keys else ''
        try:
            self.total_poi_count=float(total_poi_count)
        except:
            self.total_poi_count=0
        residential_poi_count=kwargs['residential_poi_count'] if 'residential_poi_count' in keys else ''
        try:
            self.residential_poi_count=float(residential_poi_count)
        except:
            self.residential_poi_count=0
        office_poi_count=kwargs['office_poi_count'] if 'office_poi_count' in keys else ''
        try:
            self.office_poi_count=float(office_poi_count)
        except:
            self.office_poi_count=0
        shopping_poi_count=kwargs['shopping_poi_count'] if 'shopping_poi_count' in keys else ''
        try:
            self.shopping_poi_count=float(shopping_poi_count)
        except:
            self.shopping_poi_count=0
        school_poi_count=kwargs['school_poi_count'] if 'school_poi_count' in keys else ''
        try:
            self.school_poi_count=float(school_poi_count)
        except:
            self.school_poi_count=0
        parking_poi_count=kwargs['parking_poi_count'] if 'parking_poi_count' in keys else ''
        try:
            self.parking_poi_count=float(parking_poi_count)
        except:
            self.parking_poi_count=0
        boundary_node_count=kwargs['boundary_node_count'] if 'boundary_node_count' in keys else ''
        try:
            self.boundary_node_count=float(boundary_node_count)
        except:
            self.boundary_node_count=0
        total_production=kwargs['total_production'] if 'total_production' in keys else ''
        try:
            self.total_production=float(total_production)
        except:
            self.total_production=0
        total_attraction=kwargs['total_attraction'] if 'total_attraction' in keys else ''
        try:
            self.total_attraction=float(total_attraction)
        except:
            self.total_attraction=0

class Network():
    def __init__(self):
        self.node_dict={}

        self.link_dict={}
        self.agent_dict={}
        self.demand_dict={}
        self.poi_dict={}
        self.poi_trip_dict={}
        self.zone_dict={}


        self.number_of_node=0
        self.number_of_link=0
        self.number_of_agent=0
        self.number_of_demand=0
        self.number_of_poi=0
        self.number_of_zone=0
        self.number_of_poi_type=0

        self.node_coords=[]
        self.link_coords=[]
        self.poi_coords=[]

        self.range_of_zone_ids=[]



        self.min_lat=-90
        self.max_lat=90
        self.min_lng=-180
        self.max_lng=180

    def get_avl_node_attrs(self):
        self.node_attr_dict = {
            'ctrl_type': 'int',
            'activity_type': 'str',
            'production': 'float',
            'attraction': 'float',
        }
        print('%-30s%-20s' % ('attr', 'type'))
        for k, v in self.node_attr_dict.items():
            print('%-30s%-20s' % (k, v))

    def get_avl_link_attrs(self):
        self.link_attr_dict = {
            'length': 'float',
            'lanes': 'int',
            'free_speed': 'float',
            'capacity': 'float',
            'link_type_name': 'str',
            'allowed_uses': 'str',
        }
        print('%-30s%-20s' % ('attr', 'type'))
        for k, v in self.link_attr_dict.items():
            print('%-30s%-20s' % (k, v))

    def get_avl_poi_attrs(self):
        self.poi_attr_dict = {
            'building': 'str',
            'activity_zone_id': 'int'
        }
        print('%-30s%-20s' % ('attr', 'type'))
        for k, v in self.poi_attr_dict.items():
            print('%-30s%-20s' % (k, v))

    def get_avl_range_of_zone_ids(self):
        if self.number_of_zone==0:
            print("zone.csv doesn't exist")
        else:
            print('%-20s%-20s' % ('min zone id', 'max zone id'))
            print('%-20s%-20s' % (self.range_of_zone_ids[0],self.range_of_zone_ids[1]))


from .network import readNetwork
from .network import showNetByAllMode
from .network import showNetByAutoMode
from .network import showNetByBikeMode
from .network import showNetByWalkMode
from .network import showNetByRailMode
from .network import get_num_of_nodes_by_attr
from .network import get_range_of_node_attr
from .network import get_num_of_links_by_attr
from .network import get_range_of_link_attr
from .network import get_num_of_pois_by_attr
from .network import get_info_of_zone_by_id
from .network import showNetByNodeAttr
from .network import showNetByLinkAttr
from .network import showNetByNodeProduction
from .network import showNetByNodeAttraction
from .network import showNetByLinkFreeSpeed
from .network import showNetByLinkCapacity
from .network import showNetByLinkLaneNum
from .network import showNetByPOIAttr
from .network import showNetByZonePOIAttractionDensity
from .network import showNetByZonePOIProductionDensity
from .network import showNetByZoneDemandHeatMap
from .network import showNetByZoneDemandFlow
from .network import showNetByAgentTrace

__all__=['readNetwork','showNetByAllMode','showNetByAutoMode','showNetByBikeMode','showNetByWalkMode',
         'showNetByRailMode','get_num_of_nodes_by_attr','get_range_of_node_attr','get_num_of_links_by_attr',
         'get_range_of_link_attr','get_num_of_pois_by_attr','get_info_of_zone_by_id','showNetByNodeAttr',
         'showNetByLinkAttr','showNetByNodeProduction','showNetByNodeAttraction','showNetByLinkCapacity',
         'showNetByLinkLaneNum','showNetByLinkFreeSpeed','showNetByPOIAttr','showNetByZonePOIAttractionDensity',
         'showNetByZonePOIProductionDensity','showNetByZoneDemandHeatMap','showNetByZoneDemandFlow','showNetByAgentTrace']
__version__='0.0.9'




# -*- coding: utf-8 -*-
# @Time    : 2021/4/16 19:35
# @Author  : CZY
# @File    : setup.py
# @Software: PyCharm
import setuptools
setuptools.setup(
    name='plot4gmns',
    version='0.0.9',
    author='Dr.Junhua Chen, Zanyang Cui',
    author_email='cjh@bjtu.edu.cn, zanyangcui@outlook.com',
    url='https://github.com/PariseC/plot4gmns',
    description='An open-source academic research tool for visualizing multimodal networks for transportation system modeling and optimization',
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    license='Apache License 2.0',
    packages=['plot4gmns'],
    python_requires=">=3.6.0",
    install_requires=['pandas', 'shapely','numpy','seaborn','matplotlib <=3.3.0','scipy','chardet'],
    classifiers=['License :: OSI Approved :: Apache Software License',
                 'Programming Language :: Python :: 3']
)
